import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


//-------------------------------------------------//
//                                                 //
//               CLASSE FENVIEWER                  //
//                                                 //
//-------------------------------------------------//

public class FenViewer extends JInternalFrame implements ActionListener {

    private JButton etape = new JButton(">");
    private JButton restart = new JButton("|<");
    private JButton rapide = new JButton(">>");
    private Graphe graphe;
    private AlgoFF algo;
    private ConFenViewer contenu;

    //---------------------------------------------------
    // Constructeur:
    // C dans le nom ...
    //---------------------------------------------------
    
    public FenViewer(Graphe mg){
	
	super("Viewer", 
	      true, //resizable
	      false, //closable
	      true, //maximizable
	      false);//iconifiable
	setSize(640,480);
	setLocation(0,0);

	contenu = new ConFenViewer(mg);
	this.getContentPane().add(contenu, "Center");
	
	// Ajout du Panel des boutons	
	JPanel nav=new JPanel();

	nav.add(restart);
	nav.add(etape);
	nav.add(rapide);
	getContentPane().add(nav, "South");
	
	contenu.repaint();

	setVisible(true);
	
	restart.addActionListener(this);
	etape.addActionListener(this);
	rapide.addActionListener(this);
	
	graphe=mg;
	algo=new AlgoFF(graphe);
	algo.setViewer(this);
	algo.setModeExecution(PasAPas.MODE_PAS_A_PAS);

    }

    //---------------------------------------------------
    // Fonction afficherContenu():
    // Permet de reafficher le graphe ...
    //---------------------------------------------------

    public void afficherContenu(){
	contenu.repaint();
    }
    //---------------------------------------------------
    // Fonction bloquerBoutons():
    // Permet de bloquer l'usage de certains composants  ...
    //---------------------------------------------------

    public void bloquerBoutons(){
	etape.setEnabled(false);
	restart.setEnabled(false);
	rapide.setEnabled(false);
    }
    //---------------------------------------------------
    // Fonction debloquerBoutons():
    // Permet de debloquer les composants precedement bloques
    //---------------------------------------------------

    public void debloquerBoutons(){
	etape.setEnabled(true);
	restart.setEnabled(true);
	rapide.setEnabled(true);
    }

    //---------------------------------------------------
    // Fonction actionPerformed():
    // 
    //---------------------------------------------------

    public void actionPerformed(ActionEvent e) {

	// Actions du bouton etape
    if(e.getSource()==etape)
      {
	  //System.out.println(algo.getEtatDuThread());
	  switch(algo.getEtatDuThread())
	      {
	      case 0:
		  //algo.setModeExecution(PasAPas.MODE_PAS_A_PAS);
		  algo.start();
		  bloquerBoutons();
		  break;
	      case 1:
		  bloquerBoutons();
		  algo.pas();
		  break;
	      case 2:
		  //Afficher une fenetre disant ke c fini
		  JOptionPane.showMessageDialog(this,"L'algorithme est termine\nLe flot est maximum\nflot="+graphe.getFlot(),"Fin d'algorithme",JOptionPane.PLAIN_MESSAGE);
		  break;
	      default:
		  System.out.println("(EEE) Erreur de Switch dans FenViewer");

	      }
     }
    // Actions du boutons restart
    if(e.getSource()==restart)
	{
	    graphe.faireRAZ();
	    algo=new AlgoFF(graphe);
	    algo.setViewer(this);
	    contenu.repaint();
	}
    if(e.getSource()==rapide)
      {
	  //System.out.println(algo.getEtatDuThread());
	  switch(algo.getEtatDuThread())
	      {
	      case 0:
		  algo.setModeExecution(PasAPas.MODE_CONTINU);
		  algo.start();
		  bloquerBoutons();
		  break;
	      case 1:
		  algo.setModeExecution(PasAPas.MODE_CONTINU);
		  bloquerBoutons();
		  algo.pas();
		  break;
	      case 2:
		  //Afficher une fenetre disant ke c fini
		  JOptionPane.showMessageDialog(this,"L'algorithme est termine\nLe flot est maximum\nflot="+graphe.getFlot(),"Fin d'algorithme",JOptionPane.PLAIN_MESSAGE);
		  break;
	      default:
		  System.out.println("(EEE) Erreur de Switch dans FenViewer");

	      }
     }
   }


    //---------------------------------------------------
    // Fonction maj():
    // Recreer les objets graphiques suite a une modif du graphe
    //---------------------------------------------------

  public void maj() {
    contenu.creationGrapheGraph();
    graphe.faireRAZ();
    algo=new AlgoFF(graphe);
    algo.setViewer(this);
  }

//--------------------------------------------------------------//
//                                                              //
//                   Fin de la CLASSE				//
//                                                              //
//--------------------------------------------------------------//

}

//-------------------------------------------------//
//                                                 //
//             CLASSE CONFENVIEWER                 //
//                                                 //
//-------------------------------------------------//

class ConFenViewer extends JPanel implements Observer{

 
    private Graphe graphe=null;
 
    private LinkedList listeSommetAFF = new LinkedList();
    private LinkedList listeArcAFF = new LinkedList();
    private int pasX=50;
    private int pasY=50;
    
    //---------------------------------------------------
    // Constructeur:
    // C dans le nom ...
    //---------------------------------------------------
       
    public ConFenViewer(Graphe gra) {
	graphe=gra;
	this.creationGrapheGraph();
    }
    
    //---------------------------------------------------
    // Fonction update():
    // Observer ...
    //---------------------------------------------------

    public void update(Observable o, Object obj){
	repaint();
    } 
    
    //---------------------------------------------------
    // Fonction update():
    // 
    //---------------------------------------------------

    public void update(Graphics g){
	paintComponent(g);
    }
    
    //---------------------------------------------------
    // Fonction paintComponent():
    // Permet d'afficher le graphe ...
    //---------------------------------------------------

    public void paintComponent(Graphics g) {
	super.paintComponent(g);
	
	SommetAFF sommetCourant;
	ArcAFF arcCourant;	
	int i,taille;

	// Calcul des Pas
	setPas();
	
	//g.setBackground

	// Affichage des Arcs
	taille=listeArcAFF.size();
	for(i=0;i<taille;i++)
	    {
		arcCourant=(ArcAFF)listeArcAFF.get(i);
		arcCourant.dessiner(g,pasX,pasY);
	    }
	
	// Affichage des Sommets
	taille=listeSommetAFF.size();
	for(i=0;i<taille;i++)
	    {
		sommetCourant=(SommetAFF)listeSommetAFF.get(i);
		sommetCourant.dessiner(g,pasX,pasY);
	    }

	// Affichage de l'arc retour
	drawArcRetour(g);

    }
    
    //---------------------------------------------------
    // Fonction creationGrapheGraph():
    // Permet de creer les objets graphiques ...
    //---------------------------------------------------

    public void creationGrapheGraph() {
	
	int i, taille;
	LinkedList tmpList = new LinkedList();
	SommetAFF sommAffCourant;
	ArcAFF arcAffCourant;
	listeSommetAFF.clear();
	listeArcAFF.clear();
	// Creation des SommetAFF
	tmpList.addAll(graphe.getListeSommet());
	taille=tmpList.size();
	for(i=0;i<taille;i++)
	    {
		sommAffCourant=new SommetAFF((Sommet)tmpList.get(i),true);
		listeSommetAFF.add(sommAffCourant);
	    }

	// Vidange de la liste
	tmpList.clear();

	// Creation des ArcAFF
	tmpList.addAll(graphe.getListeArc());
	taille=tmpList.size();
	for(i=0;i<taille;i++)
	    {
		arcAffCourant=new ArcAFF((Arc)tmpList.get(i),true);
		listeArcAFF.add(arcAffCourant);
	    }
    }
    
    //---------------------------------------------------
    // Fonction getListeSommetAFF():
    // Permet de recuperer la liste des SommetAFF ...
    //---------------------------------------------------

    public LinkedList getListeSommetAFF() {
	return listeSommetAFF;
    }
    
    //---------------------------------------------------
    // Fonction getListeArcAFF():
    // Permet de recuperer la liste des ArcAFF ...
    //---------------------------------------------------

    public LinkedList getListeArcAFF() {
	return listeArcAFF;
    }
    
    //---------------------------------------------------
    // Fonction setPas():
    // Permet de calculer le pas ...
    //---------------------------------------------------

    public void setPas() {
	
	int tailleFenX,tailleFenY,maxX,maxY; 
	

	tailleFenY=(int)(new Double(this.getSize().getHeight()).intValue());
	tailleFenX=(int)(new Double(this.getSize().getWidth()).intValue());
	
	// A revoir avec le graphe en argument ... :-((
	maxX=graphe.getMaxX()-graphe.getMinX()+2;
	maxY=graphe.getMaxY()-graphe.getMinY()+2;
	
	pasX=(new Integer(tailleFenX/maxX).intValue());
	pasY=(new Integer(tailleFenY/maxY).intValue());
	
    }
  
    //---------------------------------------------------
    // Fonction getPasX(), getPasY():
    // Gestion des Pas en X et Y ...
    //---------------------------------------------------

    public int getPasX() {
	return pasX;
    }
    
    public int getPasY() {
	return pasY;
    }

    //---------------------------------------------------
    // Fonction drawArcRetour():
    // Permet l'affichage de l'Arc retour ...
    //---------------------------------------------------

    public void drawArcRetour(Graphics g){

	int indexMaxX,indexMaxY,indexMinX,indexMinY;
	FontMetrics fm = g.getFontMetrics();	

	indexMaxX=graphe.getMaxX();
	indexMaxY=graphe.getMaxY();
	indexMinX=graphe.getMinX();
	indexMinY=graphe.getMinY();

	g.setColor(Color.black);

	int pointX[]={graphe.getSource().getX()*pasX-SommetAFF.getRayon(),indexMinX*pasX-50,indexMinX*pasX-50,indexMaxX*pasX+50,indexMaxX*pasX+50,graphe.getPuit().getX()*pasX+SommetAFF.getRayon()};
	int pointY[]={graphe.getSource().getY()*pasY,graphe.getSource().getY()*pasY,indexMaxY*pasY+50,indexMaxY*pasY+50,graphe.getPuit().getY()*pasY,graphe.getPuit().getY()*pasY};
	g.drawPolyline(pointX,pointY,6);

	// Affichage de la fleche et du label
	int haut=fm.getHeight()+1;
	int coorX=((indexMinX+indexMaxX)/2)*pasX;
	int coorY=indexMaxY*pasY+50;
	String str="<";
	g.drawString(str,coorX,coorY-(haut)/2+fm.getAscent());
	String lbl="Flot : "+Integer.toString(graphe.getFlot());
	int larg=fm.stringWidth(lbl) + 10;
	int milieuX=(indexMinX*pasX+indexMaxX*pasX)/2;
	int milieuY=indexMaxY*pasY+50;
	g.setColor(Color.yellow);
	g.fillRect(milieuX - larg/2 , milieuY - haut/2 , larg ,haut);
	// Dessin de la police
	g.setColor(Color.black);
	g.drawRect(milieuX - larg/2 , milieuY - haut/2 , larg  ,haut);
	g.drawString(lbl,milieuX-(larg-10)/2,(milieuY-(haut-1)/2) + fm.getAscent());

    }

//--------------------------------------------------------------//
//                                                              //
//                   Fin de la CLASSE				//
//                                                              //
//--------------------------------------------------------------//

}
