
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Attr;
//import org.xml.sax.Parser;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.util.URI;
import org.apache.xml.serialize.*;

import java.io.*;
import java.util.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

/**
 * Classe <code>GenerateXML<code> Description :
 * Cette classe est composee de trois methodes principales :
 * load()  : chargement des donnees du graphe
 * generer(): creation du document xml
 * 
 * ces methodes sont  la disposition du public  travers les methodes suivantes:
 * generateXML()
 * 
 */

public class GenerateXML {
    private Graphe graphe;
    private DocumentImpl doc;
    private FileWriter out;
    private XMLSerializer X;
    
    //---------------------------------------------------
    // Constructeur:
    // C dans le nom ...
    //---------------------------------------------------

    public GenerateXML(Graphe gr) {
	graphe = gr;
    }
    
    //---------------------------------------------------
    // Fonction generer():
    // Permet de generer un fichier XML a partir du Graphe
    //---------------------------------------------------

    public void generer(String filename) throws IOException {

        try {
	    doc = new DocumentImpl();
        }
        catch(Exception e) {
	    System.out.println("Error " + e);   
        }
        out = new FileWriter(filename);
	
	Element main,rootSommet,item,rootArc,item2,capacite,rootSource,rootPuit;
        
	//      element principale
        main = doc.createElement("graphe");
       	rootSommet = doc.createElement("sommets");
      	rootArc = doc.createElement("arcs");
	rootSource = doc.createElement("source");
	rootPuit = doc.createElement("puit");
	
       	List listeSommets = graphe.getListeSommet(); 
       	List listeArcs	 = graphe.getListeArc();
	
	//      Construction de l element Sommet      
       	for(int i = 0 ; i < listeSommets.size(); i++) {
	    
	    Sommet sommetCourant;
	    sommetCourant = (Sommet)listeSommets.get(i);
	    
	    //ajouter l'element sommet
	    item = doc.createElement("sommet");
	    
	    item.setAttribute("idSommet",Integer.toString(sommetCourant.getId()));
	    item.setAttribute("x",Integer.toString(sommetCourant.getX()));
	    item.setAttribute("y",Integer.toString(sommetCourant.getY()));
	    rootSommet.appendChild(item);
	    
	    
	    //attacher les elements crees a leur racine
            main.appendChild(rootSommet);
	    
        }//fin boucle for
        
	
	//      Construction de l element Arc
        for(int l = 0 ; l < listeArcs.size(); l++) {
   
            Arc arcCourant;
            arcCourant = (Arc)listeArcs.get(l);
	    
	    //ajouter l'element arc
            item2 = doc.createElement("arc");
            item2.setAttribute("sommetInitial",Integer.toString(arcCourant.getInitial().getId()));
            item2.setAttribute("sommetTerminal",Integer.toString(arcCourant.getTerminal().getId()));
	    item2.setAttribute("capacite",Integer.toString(arcCourant.getCapacite()));
	    
            rootArc.appendChild(item2);
	    
	    //attacher les elements crees a leur racine
            main.appendChild(rootArc);
	    
        }//fin boucle for

        // Construction de l element Source
	rootSource.setAttribute("idsource",Integer.toString(graphe.getSource().getId()));
	main.appendChild(rootSource);

	// Construction de l element Puit
	rootPuit.setAttribute("idpuit",Integer.toString(graphe.getPuit().getId()));
	main.appendChild(rootPuit);

	doc.appendChild(main);
	OutputFormat o = new OutputFormat("XML","ISO-8859-1",true);
	o.setIndent(1);
	o.setIndenting(true);
	File temp2 = new File("");
	//String chemin =	temp2.getAbsolutePath()+"\\graphe.dtd";
	//o.setDoctype(null,chemin);
	o.setDoctype(null,"graphe.dtd");
	//System.out.println("generation de la DTD");
	X = new XMLSerializer(o);
	X.setOutputCharStream(out); 
	X.serialize(doc);
	out.flush();
	
    }//fin generer()
    

    //---------------------------------------------------
    // Fonction parserXML():
    // Lecture du fichier xml
    //---------------------------------------------------

    public Graphe parserXML(String filename) {
	
	graphe.clear();
    	try {
	    DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
	    Document document = docBuilder.parse(new File(filename));

	    document.getDocumentElement().normalize();

	    // Parser les Sommets
	    NodeList listOfSommets = document.getElementsByTagName("sommet");
	    int totalSommets = listOfSommets.getLength();
	    Integer x,y,id,id1,id2,capa;
	    for(int s = 0; s <  totalSommets; s++) {
		
		Node firstSommetNode = listOfSommets.item(s);
		
		NamedNodeMap nnm = firstSommetNode.getAttributes();
		Attr attr1,attr2,attr0;
		
		attr0 = (Attr)nnm.item(0);
		id = new Integer(attr0.getNodeValue());
		attr1 = (Attr)nnm.item(1);
		x = new Integer(attr1.getNodeValue());
		attr2 = (Attr)nnm.item(2);
		y = new Integer(attr2.getNodeValue());	
		graphe.ajouterSommet(x.intValue(),y.intValue(),id.intValue());
	    }
	  
	    // Parser les Arcs
	    NodeList listOfArcs = document.getElementsByTagName("arc");
	    int totalArcs = listOfArcs.getLength();
	    Sommet initial,terminal;
	    for(int p = 0; p < totalArcs ; p++) {
		
		Node firstSommetNode = listOfArcs.item(p);
		NamedNodeMap nnm = firstSommetNode.getAttributes();
		Attr attr1a,attr2a,attr0a;
		attr0a = (Attr)nnm.item(0);
		capa = new Integer(attr0a.getNodeValue());
		attr1a = (Attr)nnm.item(1);
		id1 = new Integer(attr1a.getNodeValue());
		attr2a = (Attr)nnm.item(2);
		id2 = new Integer(attr2a.getNodeValue());
		//System.out.println("id1:"+id1+"-id2:"+id2.intValue()+"-capa:"+capa.intValue());

		initial=graphe.getSommet(id1.intValue());
		//System.out.println("Init"+initial.getId());
		terminal=graphe.getSommet(id2.intValue());
		if(initial!=null && terminal!=null)
		    {
			graphe.ajouterArc(initial,terminal,capa.intValue());
			//System.out.println("Marchage ...");
		    }
		else{System.out.println("Merdage ...");}

		//System.out.println("Foutteur de merde 3 !!!");




	    }
 	    //System.out.println("...in the rain");

	    Integer attributS, attributP;
	    // Parser l element source
	    NodeList source = document.getElementsByTagName("source");
	    Node firstSommetNode = source.item(0);
	    NamedNodeMap nnmS = firstSommetNode.getAttributes();
	    Attr attrS;
	    attrS = (Attr)nnmS.item(0);
	    attributS = new Integer(attrS.getNodeValue());
	    //System.out.println("Source : "+attributS);
	    graphe.setSource(graphe.getSommet(attributS.intValue()));

	    // Parser l element puit
	    NodeList puit = document.getElementsByTagName("puit");
	    Node firstSommetNodeP = puit.item(0);
	    NamedNodeMap nnmP = firstSommetNodeP.getAttributes();
	    Attr attrP;
	    attrP = (Attr)nnmP.item(0);
	    attributP = new Integer(attrP.getNodeValue());
	    //System.out.println("Puit : "+attributP);
	    graphe.setPuit(graphe.getSommet(attributP.intValue()));


	}
	catch (SAXParseException err)
	    {
		System.out.println ("** Parsing error" + ", line " + err.getLineNumber () + ", uri " + err.getSystemId ());
		System.out.println(" " + err.getMessage ());
		
	    }
	catch (SAXException e)
	    {
		Exception x = e.getException ();
		((x == null) ? e : x).printStackTrace ();
		
	    }
	catch (Throwable t)
	    {
		t.printStackTrace ();
	    }
	//graphe.printConsole();
	return graphe;
	
    }//fin fonction parser()
    
    
}//fin class
    

